@@ -24,6 +24,8 @@
  */
 
 
+#define INDICATOR_SERVICE_DIR "/usr/share/unity/indicators"
+
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -36,34 +38,39 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4panel/xfce-panel-plugin.h>
 #include <libindicator/indicator-object.h>
+#ifdef HAVE_LIBINDICATOR_INDICATOR_NG_H
+#include <libindicator/indicator-ng.h>
+#endif
 
 #include "indicator.h"
 #include "indicator-box.h"
 #include "indicator-button.h"
 #include "indicator-dialog.h"
 
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
-#define HAS_PANEL_49
-#endif
+#ifdef HAVE_IDO
+#include <libido/libido.h>
 #endif
 
 /* prototypes */
 static void             indicator_construct                        (XfcePanelPlugin       *plugin);
 static void             indicator_free                             (XfcePanelPlugin       *plugin);
-static gboolean         load_module                                (const gchar           *name,
-                                                                    IndicatorPlugin       *indicator);
 static void             indicator_show_about                       (XfcePanelPlugin       *plugin);
 static void             indicator_configure_plugin                 (XfcePanelPlugin       *plugin);
 static gboolean         indicator_size_changed                     (XfcePanelPlugin       *plugin,
                                                                     gint                   size);
-#ifdef HAS_PANEL_49
 static void             indicator_mode_changed                     (XfcePanelPlugin       *plugin,
                                                                     XfcePanelPluginMode    mode);
-#else
-static void             indicator_orientation_changed              (XfcePanelPlugin       *plugin,
-                                                                    GtkOrientation         orientation);
+static gboolean         indicator_load_indicator                   (IndicatorPlugin       *indicator,
+                                                                    IndicatorObject       *io,
+                                                                    const gchar           *name);
+static gboolean         indicator_load_module                      (IndicatorPlugin       *indicator,
+                                                                    const gchar           *name);
+#ifdef HAVE_LIBINDICATOR_INDICATOR_NG_H
+static gboolean         indicator_load_service                     (IndicatorPlugin       *indicator,
+                                                                    const gchar           *name);
+static void             indicator_load_services                    (IndicatorPlugin       *indicator);
 #endif
+static void             indicator_load_modules                     (IndicatorPlugin       *indicator);
 
 
 struct _IndicatorPluginClass
@@ -76,12 +83,17 @@ struct _IndicatorPlugin
 {
   XfcePanelPlugin __parent__;
 
+  gint             indicator_count;
+
   /* panel widgets */
   GtkWidget       *item;
   GtkWidget       *buttonbox;
 
   /* indicator settings */
   IndicatorConfig *config;
+
+  /* log file */
+  FILE            *logfile;
 };
 
 
@@ -102,11 +114,7 @@ indicator_class_init (IndicatorPluginClass *klass)
   plugin_class->size_changed = indicator_size_changed;
   plugin_class->about = indicator_show_about;
   plugin_class->configure_plugin = indicator_configure_plugin;
-#ifdef HAS_PANEL_49
   plugin_class->mode_changed = indicator_mode_changed;
-#else
-  plugin_class->orientation_changed = indicator_orientation_changed;
-#endif
 }
 
 
@@ -119,6 +127,12 @@ indicator_init (IndicatorPlugin *indicator)
      plugin but not internal one (loaded by "xfce4-panel" itself).
      The following lines makes only g_error critical. */
   g_log_set_always_fatal (G_LOG_LEVEL_ERROR);
+
+  indicator->indicator_count = 0;
+  indicator->item            = NULL;
+  indicator->buttonbox       = NULL;
+  indicator->config          = NULL;
+  indicator->logfile         = NULL;
 }
 
 
@@ -141,10 +155,17 @@ indicator_show_about (XfcePanelPlugin *plugin)
 {
    GdkPixbuf *icon;
 
-   const gchar *auth[] = {
-     "Mark Trompell <mark@foresightlinux.org>", "Andrzej Radecki <ndrwrdck@gmail.com>",
-     "Lionel Le Folgoc <lionel@lefolgoc.net>", "Jason Conti <jconti@launchpad.net>",
-     "Nick Schermer <nick@xfce.org>", "Evgeni Golov <evgeni@debian.org>", NULL };
+   const gchar *auth[] =
+     {
+       "Mark Trompell <mark@foresightlinux.org>",
+       "Andrzej Radecki <ndrwrdck@gmail.com>",
+       "Lionel Le Folgoc <lionel@lefolgoc.net>",
+       "Alistair Buxton <a.j.buxton@gmail.com>",
+       "Jason Conti <jconti@launchpad.net>",
+       "Nick Schermer <nick@xfce.org>",
+       "Evgeni Golov <evgeni@debian.org>",
+       NULL
+     };
 
    g_return_if_fail (XFCE_IS_INDICATOR_PLUGIN (plugin));
 
@@ -154,7 +175,7 @@ indicator_show_about (XfcePanelPlugin *plugin)
                          "license", xfce_get_license_text (XFCE_LICENSE_TEXT_GPL),
                          "version", PACKAGE_VERSION,
                          "program-name", PACKAGE_NAME,
-                         "comments", _("An indicator of something that needs your attention on the desktop"),
+                         "comments", _("Provides a panel area for Unity indicators. Indicators allow applications and system services to display their status and interact with the user."),
                          "website", "http://goodies.xfce.org/projects/panel-plugins/xfce4-indicator-plugin",
                          "copyright", _("Copyright (c) 2009-2013\n"),
                          "authors", auth, NULL);
@@ -176,7 +197,6 @@ indicator_configure_plugin (XfcePanelPlugin *plugin)
 
 
 
-#ifdef HAS_PANEL_49
 static void
 indicator_mode_changed (XfcePanelPlugin     *plugin,
                         XfcePanelPluginMode  mode)
@@ -195,49 +215,80 @@ indicator_mode_changed (XfcePanelPlugin     *plugin,
 
 
 
-#else
-static void
-indicator_orientation_changed (XfcePanelPlugin *plugin,
-                               GtkOrientation   orientation)
-{
-  IndicatorPlugin *indicator = XFCE_INDICATOR_PLUGIN (plugin);
-
-  indicator_config_set_orientation (indicator->config, orientation, GTK_ORIENTATION_HORIZONTAL);
-
-  indicator_size_changed (plugin, xfce_panel_plugin_get_size (plugin));
-}
-#endif
-
-
 static gboolean
 indicator_size_changed (XfcePanelPlugin *plugin,
                         gint             size)
 {
   IndicatorPlugin *indicator = XFCE_INDICATOR_PLUGIN (plugin);
 
-#ifdef HAS_PANEL_49
   indicator_config_set_size (indicator->config, size, xfce_panel_plugin_get_nrows (plugin));
-#else
-  indicator_config_set_size (indicator->config, size, 1);
-#endif
 
   return TRUE;
 }
 
 
+
+static void
+indicator_log_handler (const gchar    *domain,
+                       GLogLevelFlags  level,
+                       const gchar    *message,
+                       gpointer        data)
+{
+  IndicatorPlugin *indicator = XFCE_INDICATOR_PLUGIN (data);
+  gchar           *path;
+  const gchar     *prefix;
+
+  if (indicator->logfile == NULL)
+    {
+      g_mkdir_with_parents (g_get_user_cache_dir (), 0755);
+      path = g_build_filename (g_get_user_cache_dir (), "xfce4-indicator-plugin.log", NULL);
+      indicator->logfile = fopen (path, "w");
+      g_free (path);
+    }
+
+  if (indicator->logfile)
+    {
+      switch (level & G_LOG_LEVEL_MASK)
+        {
+        case G_LOG_LEVEL_ERROR:    prefix = "ERROR";    break;
+        case G_LOG_LEVEL_CRITICAL: prefix = "CRITICAL"; break;
+        case G_LOG_LEVEL_WARNING:  prefix = "WARNING";  break;
+        case G_LOG_LEVEL_MESSAGE:  prefix = "MESSAGE";  break;
+        case G_LOG_LEVEL_INFO:     prefix = "INFO";     break;
+        case G_LOG_LEVEL_DEBUG:    prefix = "DEBUG";    break;
+        default:                   prefix = "LOG";      break;
+        }
+
+      fprintf (indicator->logfile, "%-10s %-25s %s\n", prefix, domain, message);
+      fflush (indicator->logfile);
+    }
+
+  /* print log to the stdout */
+  if (level & G_LOG_LEVEL_ERROR || level & G_LOG_LEVEL_CRITICAL)
+    g_log_default_handler (domain, level, message, NULL);
+}
+
+
+
 static void
 indicator_construct (XfcePanelPlugin *plugin)
 {
   IndicatorPlugin  *indicator = XFCE_INDICATOR_PLUGIN (plugin);
-  gint              indicators_loaded = 0;
   GtkWidget        *label;
 
+  #ifdef HAVE_IDO
+  ido_init();
+  #endif
+
   xfce_panel_plugin_menu_show_configure (plugin);
   xfce_panel_plugin_menu_show_about (plugin);
 
   /* setup transation domain */
   xfce_textdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
+  /* log messages to a file */
+  g_log_set_default_handler(indicator_log_handler, plugin);
+
   /* Init some theme/icon stuff */
   gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(),
                                   INDICATOR_ICONS_DIR);
@@ -252,33 +303,12 @@ indicator_construct (XfcePanelPlugin *plugin)
   gtk_widget_show(GTK_WIDGET(indicator->buttonbox));
 
   /* load 'em */
-  if (g_file_test(INDICATOR_DIR, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))) {
-    GDir * dir = g_dir_open(INDICATOR_DIR, 0, NULL);
+  indicator_load_modules (indicator);
+#ifdef HAVE_LIBINDICATOR_INDICATOR_NG_H
+  indicator_load_services (indicator);
+#endif
 
-    const gchar * name;
-    if (indicator_config_get_mode_whitelist (indicator->config))
-      {
-        while ((name = g_dir_read_name (dir)) != NULL)
-          if (indicator_config_is_whitelisted (indicator->config, name))
-            {
-              g_debug ("Loading whitelisted module: %s", name);
-              if (load_module(name, indicator))
-                indicators_loaded++;
-            }
-      }
-    else
-      {
-        while ((name = g_dir_read_name (dir)) != NULL)
-          if (indicator_config_is_blacklisted (indicator->config, name))
-            g_debug ("Excluding blacklisted module: %s", name);
-          else if (load_module(name, indicator))
-            indicators_loaded++;
-      }
-
-    g_dir_close (dir);
-  }
-
-  if (indicators_loaded == 0) {
+  if (indicator->indicator_count == 0) {
     /* A label to allow for click through */
     indicator->item = xfce_indicator_button_new (NULL,
                                                  "<placeholder>",
@@ -306,6 +336,8 @@ entry_added (IndicatorObject * io, IndicatorObjectEntry * entry, gpointer user_d
                                                        plugin,
                                                        indicator->config);
 
+  g_debug("Entry added for io=%s", io_name);
+
   /* remove placeholder item when there are real entries to be added */
   if (indicator->item != NULL)
     {
@@ -340,33 +372,28 @@ entry_removed (IndicatorObject * io, IndicatorObjectEntry * entry, gpointer user
 
 
 static gboolean
-load_module (const gchar * name, IndicatorPlugin * indicator)
+indicator_load_indicator (IndicatorPlugin *indicator,
+                          IndicatorObject *io,
+                          const gchar     *name)
 {
-  gchar                *fullpath;
-  IndicatorObject      *io;
   GList                *entries, *entry;
   IndicatorObjectEntry *entrydata;
 
-  g_debug("Looking at Module: %s", name);
+  g_return_val_if_fail (XFCE_IS_INDICATOR_PLUGIN (indicator), 0);
   g_return_val_if_fail(name != NULL, FALSE);
-
-  if (!g_str_has_suffix(name,G_MODULE_SUFFIX))
-    return FALSE;
-
-  g_debug("Loading Module: %s", name);
+  g_debug ("Load indicator: %s", name);
 
   indicator_config_add_known_indicator (indicator->config, name);
 
-  fullpath = g_build_filename(INDICATOR_DIR, name, NULL);
-  io = indicator_object_new_from_file(fullpath);
-  g_free(fullpath);
   g_object_set_data (G_OBJECT (io), "io-name", g_strdup (name));
 
+  /* Connect to its signals */
   g_signal_connect(G_OBJECT(io), INDICATOR_OBJECT_SIGNAL_ENTRY_ADDED,
                    G_CALLBACK(entry_added), indicator);
   g_signal_connect(G_OBJECT(io), INDICATOR_OBJECT_SIGNAL_ENTRY_REMOVED,
                    G_CALLBACK(entry_removed), indicator->buttonbox);
 
+  /* Work on the entries */
   entries = indicator_object_get_entries(io);
   entry = NULL;
 
@@ -382,6 +409,150 @@ load_module (const gchar * name, IndicatorPlugin * indicator)
 }
 
 
+
+static gboolean
+indicator_load_module (IndicatorPlugin *indicator,
+                       const gchar     *name)
+{
+  gchar                *file_name;
+  IndicatorObject      *io;
+
+  g_return_val_if_fail (XFCE_IS_INDICATOR_PLUGIN (indicator), 0);
+  g_debug ("Looking at Module: %s", name);
+  g_return_val_if_fail (name != NULL, FALSE);
+
+  if (!g_str_has_suffix (name,G_MODULE_SUFFIX))
+    return FALSE;
+
+  g_debug ("Loading Module: %s", name);
+
+  file_name = g_build_filename (INDICATOR_DIR, name, NULL);
+  io = indicator_object_new_from_file (file_name);
+  g_free (file_name);
+
+  return indicator_load_indicator (indicator, io, name);
+}
+
+#ifdef HAVE_LIBINDICATOR_INDICATOR_NG_H
+static gboolean
+indicator_load_service (IndicatorPlugin *indicator,
+                        const gchar     *name)
+{
+  gchar                *fullpath;
+  IndicatorNg          *io;
+  GError               *err = NULL;
+
+  g_debug("Looking at Service: %s", name);
+  g_return_val_if_fail(name != NULL, FALSE);
+
+  g_debug("Loading Service: %s", name);
+
+  fullpath = g_build_filename(INDICATOR_SERVICE_DIR, name, NULL);
+  io = indicator_ng_new_for_profile (fullpath, "desktop", &err);
+  g_free(fullpath);
+
+  if (io != NULL)
+    {
+      indicator_load_indicator(indicator, INDICATOR_OBJECT(io), name);
+      return TRUE;
+    }
+  else
+    {
+      g_error_free (err);
+      return FALSE;
+    }
+}
+
+
+static void
+indicator_load_services (IndicatorPlugin *indicator)
+{
+  GDir        *indicators_ng_dir;
+  const gchar *io_name;
+  GError      *err = NULL;
+
+  g_return_if_fail (XFCE_IS_INDICATOR_PLUGIN (indicator));
+
+  indicators_ng_dir = g_dir_open (INDICATOR_SERVICE_DIR, 0, &err);
+
+  if (err != NULL)
+    {
+      g_warning ("%s", err->message);
+      g_error_free (err);
+
+      return;
+    }
+
+  if (indicator_config_get_mode_whitelist (indicator->config))
+    {
+      while ((io_name = g_dir_read_name (indicators_ng_dir)) != NULL)
+        if (indicator_config_is_whitelisted (indicator->config, io_name))
+          {
+            g_debug ("Loading whitelisted service: %s", io_name);
+            if (indicator_load_service (indicator, io_name))
+              indicator->indicator_count++;
+          }
+    }
+  else
+    {
+      while ((io_name = g_dir_read_name (indicators_ng_dir)) != NULL)
+        if (indicator_config_is_blacklisted (indicator->config, io_name))
+          g_debug ("Excluding blacklisted service: %s", io_name);
+        else
+          if (indicator_load_service (indicator, io_name))
+            indicator->indicator_count++;
+    }
+
+  g_dir_close (indicators_ng_dir);
+}
+#endif
+
+
+
+static void
+indicator_load_modules (IndicatorPlugin *indicator)
+{
+  GDir        *indicators_dir;
+  const gchar *io_name;
+  GError      *err = NULL;
+
+  g_return_if_fail (XFCE_IS_INDICATOR_PLUGIN (indicator));
+
+  indicators_dir = g_dir_open (INDICATOR_DIR, 0, &err);
+
+  if (err != NULL)
+    {
+      g_warning ("%s", err->message);
+      g_error_free (err);
+
+      return;
+    }
+
+  if (indicator_config_get_mode_whitelist (indicator->config))
+    {
+      while ((io_name = g_dir_read_name (indicators_dir)) != NULL)
+        if (indicator_config_is_whitelisted (indicator->config, io_name))
+          {
+            g_debug ("Loading whitelisted module: %s", io_name);
+            if (indicator_load_module(indicator, io_name))
+              indicator->indicator_count++;
+          }
+    }
+  else
+    {
+      while ((io_name = g_dir_read_name (indicators_dir)) != NULL)
+        if (indicator_config_is_blacklisted (indicator->config, io_name))
+          g_debug ("Excluding blacklisted module: %s", io_name);
+        else if (indicator_load_module(indicator, io_name))
+          indicator->indicator_count++;
+    }
+
+  g_dir_close (indicators_dir);
+}
+
+
+
+
 XfceIndicatorBox *
 indicator_get_buttonbox (IndicatorPlugin *plugin)
 {
